#!/bin/bash

#==============================================================================
# install-RPM.sh
#==============================================================================

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

if [[ $# -eq 0 ]] ; then
  echo "Usage:"
  echo "  $0 [OPTIONS] RPM1/DEB1 RPM2/DEB2 ..."
  echo "Options:"
  echo "  --update-only: only update the installed RPM/DEB"
  echo "  --update: update or install the RPM/DEB"
  echo "  (none): only install RPM/DEB if not installed yet"
  echo "Example:"
  echo "  $0 graphviz nodejs"
  exit 1
fi

case $1 in
  --update-only) DO=update ; shift ;;
  --update) DO=install_or_update ; shift ;;
  *) DO=install ;;
esac

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/functions.sh
export started_by_rpm

. $(dirname $0)/functions/fun-repoedit.sh
export enable_apt_repo restore_apt_repo

#------------------------------------------------------------------------------
# execute
#------------------------------------------------------------------------------

# if install-RPM.sh was executed by a packaging tool we can't run packaging tools
started_by_rpm $$ && exit 0

#..............................................................................
# find operating system and its packaging tools
#..............................................................................

if [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] ; then
  unset PERL5LIB
  INSTALLER=apt-get ; YES=-y ; unset ZES
elif [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%r} ]] ; then
  INSTALLER=yum ; YES=-y ; unset ZES
else
  echo "$0: operating system not supported: $(uname -a)"
  exit 1
fi

#..............................................................................
# enable or disable repos depending on envvars ENABLE_REPO and DISABLE_REPO
#..............................................................................

unset EPAGES_APT_REPO_BACKUP CLEAN_ALL

for i in $ENABLE_REPO ; do
  if [[ $INSTALLER = yum ]] ; then
    YES="$YES --enablerepo $i"
  elif [[ $INSTALLER = apt-get ]] ; then
    backup=$(enable_apt_repo 1 ${i/base/setup})
    [[ -n $backup ]] && EPAGES_APT_REPO_BACKUP=$backup
  fi
done
for i in $DISABLE_REPO ; do
  if [[ $INSTALLER = yum ]] ; then
    YES="$YES --disablerepo $i"
  elif [[ $INSTALLER = apt-get ]] ; then
    backup=$(enable_apt_repo 0 ${i/base/setup})
    [[ -n $backup ]] && EPAGES_APT_REPO_BACKUP=$backup
  fi
done

#..............................................................................
# update packages
#..............................................................................

for RPM in $@ ; do
  IS_INSTALLED=
  if [[ $INSTALLER = apt-get ]] ; then
    dpkg --status $RPM >/dev/null 2>&1 && IS_INSTALLED=1
  else
    rpm --quiet -q $RPM >/dev/null 2>&1 && IS_INSTALLED=1
  fi
  if [[ $IS_INSTALLED ]] ; then
    [[ $DO = install ]] && continue
    # workaround: sometimes file conflicts occur
    if [[ $INSTALLER = yum ]] ; then
      yum check-update $YES 2>/dev/null | awk "/^$RPM\./{exit 123}"
      [[ $? = 123 ]] || continue # up-to-date
      case $RPM in
        epages-j|epages-nodejs)
          rpm -e --nodeps $RPM ; [[ $INSTALLER = yum ]] && CLEAN_ALL=1 ;;
        epages-*)
          true ;;
        *)
          rpm -e --nodeps $RPM ; [[ $INSTALLER = yum ]] && CLEAN_ALL=1 ;;
      esac
    fi
  else
    [[ $DO = update ]] && continue
  fi
  echo "$INSTALLER $YES install $ZES $RPM"
  eval $INSTALLER $YES install $ZES $RPM || exit 1
done

#..............................................................................
# restore repositories
#..............................................................................

if [[ $INSTALLER = apt-get && -n $EPAGES_APT_REPO_BACKUP ]] ; then
  restore_apt_repo $EPAGES_APT_REPO_BACKUP
elif [[ $INSTALLER = yum && -n $CLEAN_ALL ]] ; then
  yum clean all
fi

exit 0
